import { readable, writable } from "svelte/store";

export const API_BASE_URL = "api";
const POLL_FOR_UPDATES = true;
const POLL_INTERVAL = 2500;
const REFRESH_QUERY_STRING_INTERVAL = 5000;

export const rooms = readable([], function start(set) {
  let fetchLock = false;
  const fetchRooms = async () => {
    if (POLL_FOR_UPDATES && !fetchLock) {
      fetchLock = true;
      try {
        const response = await fetch(API_BASE_URL + "/attachedRooms");
        set(await response.json());
      } catch (error) {
        console.error("Could not fetch attachedRooms because of error:", error);
      }
      fetchLock = false;
    }
  };
  fetchRooms();
  const interval = setInterval(fetchRooms, POLL_INTERVAL);

  return function stop() {
    clearInterval(interval);
  };
});

export const roomsToDetachFrom = writable([]);
roomsToDetachFrom.subscribe(async (rooms) => {
  if (rooms.length === 0) return;
  for (const uid of rooms) {
    console.log("Trying to detach from", uid);
    try {
      const response = await fetch(API_BASE_URL + "/detach", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ uid }),
      });
      await response.json();
    } catch (error) {
      console.error(
        "Could not unsubscribe from",
        uid,
        "because of error:",
        error
      );
    }
  }
  roomsToDetachFrom.update((rooms) => []);
});

export const randomQueryString = readable("?random=init", function start(set) {
  let interval = setInterval(() => {
    set("?random=" + Math.random().toString(36).substring(7));
  }, REFRESH_QUERY_STRING_INTERVAL);
  return function stop() {
    clearInterval(interval);
  };
});
